@@ -53,6 +53,7 @@ pub enum ControlFlowGraphNode {
     EnumVariant {
         span: Span,
         variant_name: String,
+        is_public: bool,
     },
     MethodDeclaration {
         span: Span,
@@ -91,15 +92,6 @@ impl std::convert::From<&TypedAstNode> for ControlFlowGraphNode {
     }
 }
 
-impl std::convert::From<&TypedEnumVariant> for ControlFlowGraphNode {
-    fn from(other: &TypedEnumVariant) -> Self {
-        ControlFlowGraphNode::EnumVariant {
-            variant_name: other.name.as_str().to_string(),
-            span: other.span.clone(),
-        }
-    }
-}
-
 impl std::convert::From<&TypedStructField> for ControlFlowGraphNode {
     fn from(other: &TypedStructField) -> Self {
         ControlFlowGraphNode::StructField {
@@ -145,3 +137,16 @@ impl ControlFlowGraph {
         println!("{:?}", Dot::with_config(&self.graph, &[]));
     }
 }
+
+impl ControlFlowGraphNode {
+    pub(crate) fn from_enum_variant(
+        other: &TypedEnumVariant,
+        is_public: bool,
+    ) -> ControlFlowGraphNode {
+        ControlFlowGraphNode::EnumVariant {
+            variant_name: other.name.as_str().to_string(),
+            span: other.span.clone(),
+            is_public,
+        }
+    }
+}
