@@ -4,6 +4,7 @@ use crate::{
     parser::Rule,
     style::{to_screaming_snake_case, to_snake_case, to_upper_camel_case},
     type_engine::*,
+    VariableDeclaration,
 };
 use sway_types::{ident::Ident, span::Span};
 
@@ -56,6 +57,15 @@ pub(crate) fn err<T>(warnings: Vec<CompileWarning>, errors: Vec<CompileError>) -
     }
 }
 
+pub(crate) fn infallible<T>(res: CompileResult<T>) -> T {
+    match res.value {
+        Some(x) if res.errors.is_empty() && res.warnings.is_empty() => x,
+        _ => {
+            panic!("Internal compiler error: called `infallible` on a fallible compile result. Please report this bug: github.com/FuelLabs/Sway/issues")
+        }
+    }
+}
+
 /// Denotes a recovered or non-error state
 pub(crate) fn ok<T>(
     value: T,
@@ -69,6 +79,25 @@ pub(crate) fn ok<T>(
     }
 }
 
+/// Acts as the result of parsing `Declaration`s, `Expression`s, etc.
+/// Some `Expression`s need to be able to create `VariableDeclaration`s,
+/// so this struct is used to "bubble up" those declarations to a viable
+/// place in the AST.
+#[derive(Debug, Clone)]
+pub struct ParserLifter<T> {
+    pub var_decls: Vec<VariableDeclaration>,
+    pub value: T,
+}
+
+impl<T> ParserLifter<T> {
+    pub(crate) fn empty(value: T) -> Self {
+        ParserLifter {
+            var_decls: vec![],
+            value,
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct CompileResult<T> {
     pub value: Option<T>,
@@ -245,6 +274,7 @@ pub enum Warning {
     ShadowingReservedRegister {
         reg_name: Ident,
     },
+    MatchExpressionUnreachableArm,
 }
 
 impl fmt::Display for Warning {
@@ -350,6 +380,7 @@ impl fmt::Display for Warning {
                 "This register declaration shadows the reserved register, \"{}\".",
                 reg_name
             ),
+            MatchExpressionUnreachableArm => write!(f, "This match arm is unreachable."),
         }
     }
 }
@@ -381,8 +412,6 @@ pub enum CompileError {
     },
     #[error("Unimplemented feature: {0}")]
     Unimplemented(&'static str, Span),
-    #[error("pattern matching algorithm failure on: {0}")]
-    PatternMatchingAlgorithmFailure(&'static str, Span),
     #[error("{0}")]
     TypeError(TypeError),
     #[error("Error parsing input: expected {err:?}")]
@@ -668,8 +697,8 @@ pub enum CompileError {
     },
     #[error("Unknown opcode: \"{op_name}\".")]
     UnrecognizedOp { op_name: Ident, span: Span },
-    #[error("Unknown type \"{ty}\".")]
-    TypeMustBeKnown { ty: String, span: Span },
+    #[error("Generic type \"{ty}\" was unable to be inferred. Insufficient type information provided. Try annotating its type.")]
+    UnableToInferGeneric { ty: String, span: Span },
     #[error("The value \"{val}\" is too large to fit in this 6-bit immediate spot.")]
     Immediate06TooLarge { val: u64, span: Span },
     #[error("The value \"{val}\" is too large to fit in this 12-bit immediate spot.")]
@@ -811,6 +840,11 @@ pub enum CompileError {
          "
     )]
     MatchWrongType { expected: TypeId, span: Span },
+    #[error("Non-exhaustive match expression. Missing patterns {missing_patterns}")]
+    MatchExpressionNonExhaustive {
+        missing_patterns: String,
+        span: Span,
+    },
     #[error("Impure function called inside of pure function. Pure functions can only call other pure functions. Try making the surrounding function impure by prepending \"impure\" to the function declaration.")]
     PureCalledImpure { span: Span },
     #[error("Impure function inside of non-contract. Contract storage is only accessible from contracts.")]
@@ -845,6 +879,8 @@ pub enum CompileError {
         trait_name: String,
         span: Span,
     },
+    #[error("Cannot use `if let` on a non-enum type.")]
+    IfLetNonEnum { span: Span },
     #[error(
         "Contract ABI method parameter \"{param_name}\" is set multiple times for this contract ABI method call"
     )]
@@ -1006,7 +1042,7 @@ impl CompileError {
             InvalidAssemblyMismatchedReturn { span, .. } => span,
             UnknownEnumVariant { span, .. } => span,
             UnrecognizedOp { span, .. } => span,
-            TypeMustBeKnown { span, .. } => span,
+            UnableToInferGeneric { span, .. } => span,
             Immediate06TooLarge { span, .. } => span,
             Immediate12TooLarge { span, .. } => span,
             Immediate18TooLarge { span, .. } => span,
@@ -1047,8 +1083,8 @@ impl CompileError {
             ShadowsOtherSymbol { span, .. } => span,
             StarImportShadowsOtherSymbol { span, .. } => span,
             MatchWrongType { span, .. } => span,
+            MatchExpressionNonExhaustive { span, .. } => span,
             NotAnEnum { span, .. } => span,
-            PatternMatchingAlgorithmFailure(_, span) => span,
             PureCalledImpure { span, .. } => span,
             ImpureInNonContract { span, .. } => span,
             IntegerTooLarge { span, .. } => span,
@@ -1059,6 +1095,7 @@ impl CompileError {
             NameDefinedMultipleTimesForTrait { span, .. } => span,
             SupertraitImplMissing { span, .. } => span,
             SupertraitImplRequired { span, .. } => span,
+            IfLetNonEnum { span, .. } => span,
             ContractCallParamRepeated { span, .. } => span,
             UnrecognizedContractParam { span, .. } => span,
             CallParamForNonContractCallMethod { span, .. } => span,
