@@ -1,14 +1,15 @@
-use std::collections::HashMap;
+use fuel_crypto::Hasher;
+use std::collections::{BTreeMap, HashMap};
 use std::iter::FromIterator;
 
 use crate::{
     asm_generation::from_ir::TypeAnalyzer,
     parse_tree::{AsmOp, AsmRegister, LazyOp, Literal, Visibility},
-    semantic_analysis::{ast_node::TypedCodeBlock, ast_node::*, *},
+    semantic_analysis::{ast_node::*, *},
     type_engine::*,
 };
 
-use sway_types::{ident::Ident, span::Span};
+use sway_types::{ident::Ident, span::Span, state::StateIndex};
 
 use sway_ir::*;
 
@@ -202,6 +203,7 @@ fn compile_declarations(
             | TypedDeclaration::EnumDeclaration(_)
             | TypedDeclaration::VariableDeclaration(_)
             | TypedDeclaration::Reassignment(_)
+            | TypedDeclaration::StorageReassignment(_)
             | TypedDeclaration::AbiDeclaration(_)
             | TypedDeclaration::GenericTypeForFunctionScope { .. }
             | TypedDeclaration::ErrorRecovery => (),
@@ -215,7 +217,7 @@ fn compile_declarations(
 #[derive(Clone, Default)]
 struct StructSymbolMap {
     aggregate_names: HashMap<String, Aggregate>,
-    aggregate_symbols: HashMap<Aggregate, HashMap<String, u64>>,
+    aggregate_symbols: HashMap<Aggregate, BTreeMap<String, u64>>,
 }
 
 impl StructSymbolMap {
@@ -223,7 +225,7 @@ impl StructSymbolMap {
         &mut self,
         name: String,
         aggregate: Aggregate,
-        symbols: Option<HashMap<String, u64>>,
+        symbols: Option<BTreeMap<String, u64>>,
     ) -> Result<(), String> {
         match self.aggregate_names.insert(name, aggregate) {
             None => Ok(()),
@@ -256,7 +258,7 @@ fn create_struct_aggregate(
     context: &mut Context,
     struct_names: &mut StructSymbolMap,
     name: String,
-    fields: Vec<OwnedTypedStructField>,
+    fields: Vec<TypedStructField>,
 ) -> Result<Aggregate, String> {
     let (field_types, syms): (Vec<_>, Vec<_>) = fields
         .into_iter()
@@ -276,8 +278,10 @@ fn create_struct_aggregate(
     struct_names.add_aggregate_symbols(
         name,
         aggregate,
-        Some(HashMap::from_iter(
-            syms.into_iter().enumerate().map(|(n, sym)| (sym, n as u64)),
+        Some(BTreeMap::from_iter(
+            syms.into_iter()
+                .enumerate()
+                .map(|(n, sym)| (sym.to_string(), n as u64)),
         )),
     )?;
 
@@ -302,22 +306,14 @@ fn compile_enum_decl(
         return Err("Unable to compile generic enums.".into());
     }
 
-    create_enum_aggregate(
-        context,
-        struct_names,
-        name.as_str().to_owned(),
-        variants
-            .into_iter()
-            .map(|tev| tev.as_owned_typed_enum_variant())
-            .collect(),
-    )
+    create_enum_aggregate(context, struct_names, name.as_str().to_owned(), variants)
 }
 
 fn create_enum_aggregate(
     context: &mut Context,
     struct_names: &mut StructSymbolMap,
     name: String,
-    variants: Vec<OwnedTypedEnumVariant>,
+    variants: Vec<TypedEnumVariant>,
 ) -> Result<Aggregate, String> {
     // Create the enum aggregate first.  NOTE: single variant enums don't need an aggregate but are
     // getting one here anyway.  They don't need to be a tagged union either.
@@ -496,6 +492,11 @@ struct FnCompiler {
     type_analyzer: TypeAnalyzer,
 }
 
+pub enum StateAccessType {
+    Read,
+    Write,
+}
+
 impl FnCompiler {
     fn new(
         context: &mut Context,
@@ -552,6 +553,14 @@ impl FnCompiler {
                         TypedDeclaration::Reassignment(tr) => {
                             self.compile_reassignment(context, tr, span_md_idx)
                         }
+                        TypedDeclaration::StorageReassignment(tr) => self
+                            .compile_storage_reassignment(
+                                context,
+                                &tr.fields,
+                                &tr.ix,
+                                &tr.rhs,
+                                span_md_idx,
+                            ),
                         TypedDeclaration::ImplTrait { span, .. } => {
                             // XXX What if I ignore the trait implementation???  Potentially since
                             // we currently inline everything and below we 'recreate' the functions
@@ -595,8 +604,9 @@ impl FnCompiler {
     ) -> Result<Value, String> {
         let span_md_idx = MetadataIndex::from_span(context, &ast_expr.span);
         match ast_expr.expression {
-            TypedExpressionVariant::Literal(l) =>
-                Ok(convert_literal_to_value(context, &l, span_md_idx)),
+            TypedExpressionVariant::Literal(l) => {
+                Ok(convert_literal_to_value(context, &l, span_md_idx))
+            }
             TypedExpressionVariant::FunctionApplication {
                 name,
                 arguments,
@@ -645,10 +655,9 @@ impl FnCompiler {
                 prefix,
                 field_to_access,
                 resolved_type_of_parent,
-                field_to_access_span,
                 ..
             } => {
-                let span_md_idx = MetadataIndex::from_span(context, &field_to_access_span);
+                let span_md_idx = MetadataIndex::from_span(context, &field_to_access.span);
                 self.compile_struct_field_expr(
                     context,
                     *prefix,
@@ -663,36 +672,37 @@ impl FnCompiler {
                 contents,
                 ..
             } => self.compile_enum_expr(context, enum_decl, tag, contents),
-            TypedExpressionVariant::EnumArgAccess {
-                //Prefix: Box<TypedExpression>,
-                //Arg_num_to_access: usize,
-                //Resolved_type_of_parent: TypeId,
-                ..
-            } => Err("enum arg access".into()),
-            TypedExpressionVariant::Tuple {
-               fields
-            } => self.compile_tuple_expr(context, fields, span_md_idx),
+            TypedExpressionVariant::IfLet { .. } => Err("if let expression ".into()),
+            TypedExpressionVariant::Tuple { fields } => {
+                self.compile_tuple_expr(context, fields, span_md_idx)
+            }
             TypedExpressionVariant::TupleElemAccess {
                 prefix,
                 elem_to_access_num: idx,
                 elem_to_access_span: span,
                 resolved_type_of_parent: tuple_type,
-            } => self.compile_tuple_elem_expr( context, *prefix, tuple_type, idx, span),
+            } => self.compile_tuple_elem_expr(context, *prefix, tuple_type, idx, span),
             TypedExpressionVariant::AbiCast { span, .. } => {
                 let span_md_idx = MetadataIndex::from_span(context, &span);
                 Ok(Constant::get_unit(context, span_md_idx))
             }
+            TypedExpressionVariant::StorageAccess(access) => {
+                let span_md_idx = MetadataIndex::from_span(context, &access.span());
+                self.compile_storage_access(context, &access.fields, &access.ix, span_md_idx)
+            }
             TypedExpressionVariant::SizeOf { variant } => {
                 match variant {
                     SizeOfVariant::Type(type_id) => {
                         let ir_type = convert_resolved_typeid_no_span(
-                            context, &mut self.struct_names, &type_id
+                            context,
+                            &mut self.struct_names,
+                            &type_id,
                         )?;
                         Ok(Constant::get_uint(
                             context,
                             64,
                             self.type_analyzer.ir_type_size_in_bytes(context, &ir_type),
-                            None
+                            None,
                         ))
                     }
                     SizeOfVariant::Val(exp) => {
@@ -700,7 +710,7 @@ impl FnCompiler {
                             context,
                             &mut self.struct_names,
                             &exp.return_type,
-                            &exp.span
+                            &exp.span,
                         )?;
 
                         // Compile the expression in case of side-effects but ignore its value.
@@ -710,11 +720,11 @@ impl FnCompiler {
                             context,
                             64,
                             self.type_analyzer.ir_type_size_in_bytes(context, &ir_type),
-                            None
+                            None,
                         ))
                     }
                 }
-            },
+            }
         }
     }
 
@@ -1012,7 +1022,11 @@ impl FnCompiler {
             .get(name)
             .and_then(|local_name| self.function.get_local_ptr(context, local_name))
         {
-            let ptr_val = self.current_block.ins(context).get_ptr(ptr, span_md_idx);
+            let ptr_ty = *ptr.get_type(context);
+            let ptr_val = self
+                .current_block
+                .ins(context)
+                .get_ptr(ptr, ptr_ty, 0, span_md_idx);
             Ok(if ptr.is_aggregate_ptr(context) {
                 ptr_val
             } else {
@@ -1070,7 +1084,11 @@ impl FnCompiler {
             .new_local_ptr(context, local_name, return_type, is_mutable.into(), None)
             .map_err(|ir_error| ir_error.to_string())?;
 
-        let ptr_val = self.current_block.ins(context).get_ptr(ptr, span_md_idx);
+        let ptr_ty = *ptr.get_type(context);
+        let ptr_val = self
+            .current_block
+            .ins(context)
+            .get_ptr(ptr, ptr_ty, 0, span_md_idx);
         self.current_block
             .ins(context)
             .store(ptr_val, init_val, span_md_idx);
@@ -1131,49 +1149,24 @@ impl FnCompiler {
 
         if ast_reassignment.lhs.len() == 1 {
             // A non-aggregate; use a `store`.
-            let ptr_val = self.current_block.ins(context).get_ptr(ptr, span_md_idx);
+            let ptr_ty = *ptr.get_type(context);
+            let ptr_val = self
+                .current_block
+                .ins(context)
+                .get_ptr(ptr, ptr_ty, 0, span_md_idx);
             self.current_block
                 .ins(context)
                 .store(ptr_val, reassign_val, span_md_idx);
         } else {
-            // An aggregate.  Iterate over the field names from the left hand side and collect
-            // field indices.
-            let field_idcs = ast_reassignment.lhs[1..]
-                .iter()
-                .fold(
-                    Ok((Vec::new(), *ptr.get_type(context))),
-                    |acc, field_name| {
-                        // Make sure we have an aggregate to index into.
-                        acc.and_then(|(mut fld_idcs, ty)| match ty {
-                            Type::Struct(aggregate) => {
-                                // Get the field index and also its type for the next iteration.
-                                match self
-                                    .struct_names
-                                    .get_aggregate_index(&aggregate, field_name.name.as_str())
-                                {
-                                    None => Err(format!(
-                                        "Unknown field name {} for struct ???",
-                                        field_name.name.as_str()
-                                    )),
-                                    Some(field_idx) => {
-                                        let field_type = context.aggregates[aggregate.0]
-                                            .field_types()
-                                            [field_idx as usize];
-
-                                        // Save the field index.
-                                        fld_idcs.push(field_idx);
-                                        Ok((fld_idcs, field_type))
-                                    }
-                                }
-                            }
-                            _otherwise => {
-                                Err("Reassignment with multiple accessors to non-aggregate.".into())
-                            }
-                        })
-                    },
-                )?
-                .0;
-
+            // An aggregate.
+            let field_idcs = self.get_indices_for_struct_access(
+                context,
+                *ptr.get_type(context),
+                &ast_reassignment.lhs[1..]
+                    .iter()
+                    .map(|x| x.name.clone())
+                    .collect::<Vec<Ident>>(),
+            )?;
             let ty = match ptr.get_type(context) {
                 Type::Struct(aggregate) => *aggregate,
                 _otherwise => {
@@ -1181,7 +1174,11 @@ impl FnCompiler {
                 }
             };
 
-            let ptr_val = self.current_block.ins(context).get_ptr(ptr, span_md_idx);
+            let ptr_ty = *ptr.get_type(context);
+            let ptr_val = self
+                .current_block
+                .ins(context)
+                .get_ptr(ptr, ptr_ty, 0, span_md_idx);
             self.current_block.ins(context).insert_value(
                 ptr_val,
                 ty,
@@ -1198,6 +1195,57 @@ impl FnCompiler {
 
     // ---------------------------------------------------------------------------------------------
 
+    fn compile_storage_reassignment(
+        &mut self,
+        context: &mut Context,
+        fields: &[TypeCheckedStorageReassignDescriptor],
+        ix: &StateIndex,
+        rhs: &TypedExpression,
+        span_md_idx: Option<MetadataIndex>,
+    ) -> Result<Value, String> {
+        // Compile the RHS into a value
+        let rhs = self.compile_expression(context, rhs.clone())?;
+
+        // Get the type of the storage field that is being accessed
+        let storage_field_type = convert_resolved_typeid_no_span(
+            context,
+            &mut self.struct_names,
+            &fields.first().expect("guaranteed by grammar").r#type,
+        )?;
+
+        // Get the type of the access which can be a subfield
+        let access_type = convert_resolved_typeid_no_span(
+            context,
+            &mut self.struct_names,
+            &fields.last().expect("guaranteed by grammar").r#type,
+        )?;
+
+        // Get the list of indices used to access the storage field. This will be empty
+        // if the storage field type is not a struct.
+        let field_idcs = self.get_indices_for_struct_access(
+            context,
+            storage_field_type,
+            &fields[1..]
+                .iter()
+                .map(|x| x.name.clone())
+                .collect::<Vec<Ident>>(),
+        )?;
+
+        // Do the actual work. This is a recursive function because we want to drill down
+        // to store each primitive type in the storage field in its own storage slot.
+        self.compile_storage_read_or_write(
+            context,
+            &StateAccessType::Write,
+            ix,
+            field_idcs,
+            &access_type,
+            &Some(rhs),
+            span_md_idx,
+        )
+    }
+
+    // ---------------------------------------------------------------------------------------------
+
     fn compile_array_expr(
         &mut self,
         context: &mut Context,
@@ -1345,7 +1393,7 @@ impl FnCompiler {
         &mut self,
         context: &mut Context,
         ast_struct_expr: TypedExpression,
-        ast_field: OwnedTypedStructField,
+        ast_field: TypedStructField,
         _ast_parent_type: TypeId,
         span_md_idx: Option<MetadataIndex>,
     ) -> Result<Value, String> {
@@ -1368,7 +1416,7 @@ impl FnCompiler {
 
         let field_idx = self
             .struct_names
-            .get_aggregate_index(&aggregate, &ast_field.name)
+            .get_aggregate_index(&aggregate, ast_field.name.as_str())
             .ok_or_else(|| format!("Unknown field name {} in struct ???", ast_field.name))?;
 
         Ok(self.current_block.ins(context).extract_value(
@@ -1507,6 +1555,53 @@ impl FnCompiler {
 
     // ---------------------------------------------------------------------------------------------
 
+    fn compile_storage_access(
+        &mut self,
+        context: &mut Context,
+        fields: &[TypeCheckedStorageAccessDescriptor],
+        ix: &StateIndex,
+        span_md_idx: Option<MetadataIndex>,
+    ) -> Result<Value, String> {
+        // Get the type of the storage field that is being accessed
+        let storage_field_type = convert_resolved_typeid_no_span(
+            context,
+            &mut self.struct_names,
+            &fields.first().expect("guaranteed by grammar").r#type,
+        )?;
+
+        // Get the type of the access which can be a subfield
+        let access_type = convert_resolved_typeid_no_span(
+            context,
+            &mut self.struct_names,
+            &fields.last().expect("guaranteed by grammar").r#type,
+        )?;
+
+        // Get the list of indices used to access the storage field. This will be empty
+        // if the storage field type is not a struct.
+        let field_idcs = self.get_indices_for_struct_access(
+            context,
+            storage_field_type,
+            &fields[1..]
+                .iter()
+                .map(|x| x.name.clone())
+                .collect::<Vec<Ident>>(),
+        )?;
+
+        // Do the actual work. This is a recursive function because we want to drill down
+        // to load each primitive type in the storage field in its own storage slot.
+        self.compile_storage_read_or_write(
+            context,
+            &StateAccessType::Read,
+            ix,
+            field_idcs,
+            &access_type,
+            &None,
+            span_md_idx,
+        )
+    }
+
+    // ---------------------------------------------------------------------------------------------
+
     fn compile_asm_expr(
         &mut self,
         context: &mut Context,
@@ -1562,6 +1657,250 @@ impl FnCompiler {
             whole_block_span_md_idx,
         ))
     }
+
+    // -------------------------------------------------------------------------------------------------
+    // Utils
+
+    #[allow(clippy::too_many_arguments)]
+    fn compile_storage_read_or_write(
+        &mut self,
+        context: &mut Context,
+        access_type: &StateAccessType,
+        ix: &StateIndex,
+        indices: Vec<u64>,
+        r#type: &Type,
+        rhs: &Option<Value>,
+        span_md_idx: Option<MetadataIndex>,
+    ) -> Result<Value, String> {
+        Ok(match r#type {
+            Type::Struct(aggregate) => {
+                let mut struct_val =
+                    Constant::get_undef(context, Type::Struct(*aggregate), span_md_idx);
+
+                for (_, aggregate_index) in self
+                    .struct_names
+                    .aggregate_symbols
+                    .get(aggregate)
+                    .ok_or("aggregate symbol not found")?
+                    .clone()
+                {
+                    // Recurse.
+                    // The base case is for primitive types that fit in a single storage slot.
+                    let field_type =
+                        context.aggregates[aggregate.0].field_types()[aggregate_index as usize];
+                    let mut new_indices = indices.clone();
+                    new_indices.push(aggregate_index);
+
+                    match access_type {
+                        StateAccessType::Read => {
+                            let val_to_insert = self.compile_storage_read_or_write(
+                                context,
+                                access_type,
+                                ix,
+                                new_indices,
+                                &field_type,
+                                rhs,
+                                span_md_idx,
+                            )?;
+
+                            //  Insert the loaded value to the aggregate at the given index
+                            struct_val = self.current_block.ins(context).insert_value(
+                                struct_val,
+                                *aggregate,
+                                val_to_insert,
+                                vec![aggregate_index],
+                                span_md_idx,
+                            );
+                        }
+                        StateAccessType::Write => {
+                            // Extract the value from the aggregate at the given index
+                            let rhs = self.current_block.ins(context).extract_value(
+                                rhs.expect("expecting a rhs for write"),
+                                *aggregate,
+                                vec![aggregate_index],
+                                span_md_idx,
+                            );
+
+                            self.compile_storage_read_or_write(
+                                context,
+                                access_type,
+                                ix,
+                                new_indices,
+                                &field_type,
+                                &Some(rhs),
+                                span_md_idx,
+                            )?;
+                        }
+                    }
+                }
+                struct_val
+            }
+            Type::Bool | Type::Uint(_) | Type::B256 => {
+                // Calculate the storage location hash for the given field
+                let mut storage_slot_to_hash = format!(
+                    "{}{}",
+                    sway_utils::constants::STORAGE_DOMAIN_SEPARATOR,
+                    ix.to_usize()
+                );
+                for ix in &indices {
+                    storage_slot_to_hash = format!("{}_{}", storage_slot_to_hash, ix);
+                }
+                let hashed_storage_slot = Hasher::hash(storage_slot_to_hash);
+
+                // New name for the key
+                let mut key_name = format!("{}{}", "key_for_", ix.to_usize());
+                for ix in &indices {
+                    key_name = format!("{}_{}", key_name, ix);
+                }
+                let alias_key_name = match self.symbol_map.get(key_name.as_str()) {
+                    None => key_name.clone(),
+                    Some(shadowed_key_name) => format!("{}_", shadowed_key_name),
+                };
+                self.symbol_map.insert(alias_key_name.clone(), key_name);
+
+                // Local pointer for the key
+                let key_ptr = self
+                    .function
+                    .new_local_ptr(context, alias_key_name, Type::B256, true, None)
+                    .map_err(|ir_error| ir_error.to_string())?;
+
+                // Const value for the key from the hash
+                let const_key = convert_literal_to_value(
+                    context,
+                    &Literal::B256(hashed_storage_slot.into()),
+                    span_md_idx,
+                );
+
+                // Convert the key pointer to a value using get_ptr
+                let key_ptr_ty = *key_ptr.get_type(context);
+                let key_ptr_val =
+                    self.current_block
+                        .ins(context)
+                        .get_ptr(key_ptr, key_ptr_ty, 0, span_md_idx);
+
+                // Store the const hash value to the key pointer value
+                self.current_block
+                    .ins(context)
+                    .store(key_ptr_val, const_key, span_md_idx);
+
+                match r#type {
+                    Type::Uint(_) | Type::Bool => {
+                        // These types fit in a word. use state_store_word/state_load_word
+                        match access_type {
+                            StateAccessType::Read => self
+                                .current_block
+                                .ins(context)
+                                .state_load_word(key_ptr_val, span_md_idx),
+                            StateAccessType::Write => {
+                                self.current_block.ins(context).state_store_word(
+                                    rhs.expect("expecting a rhs for write"),
+                                    key_ptr_val,
+                                    span_md_idx,
+                                );
+                                rhs.expect("expecting a rhs for write")
+                            }
+                        }
+                    }
+                    Type::B256 => {
+                        // B256 requires 4 words. Use state_load_quad_word/state_store_quad_word
+                        // First, create a name for the value to load from or store to
+                        let mut value_name = format!("{}{}", "val_for_", ix.to_usize());
+                        for ix in &indices {
+                            value_name = format!("{}_{}", value_name, ix);
+                        }
+                        let alias_value_name = match self.symbol_map.get(value_name.as_str()) {
+                            None => value_name.clone(),
+                            Some(shadowed_value_name) => format!("{}_", shadowed_value_name),
+                        };
+                        self.symbol_map.insert(value_name, alias_value_name.clone());
+
+                        // Local pointer to hold the B256
+                        let value_ptr = self
+                            .function
+                            .new_local_ptr(context, alias_value_name, *r#type, true, None)
+                            .map_err(|ir_error| ir_error.to_string())?;
+
+                        // Convert the local pointer created to a value using get_ptr
+                        let value_ptr_val = self.current_block.ins(context).get_ptr(
+                            value_ptr,
+                            *r#type,
+                            0,
+                            span_md_idx,
+                        );
+
+                        match access_type {
+                            StateAccessType::Read => {
+                                self.current_block.ins(context).state_load_quad_word(
+                                    value_ptr_val,
+                                    key_ptr_val,
+                                    span_md_idx,
+                                );
+                                value_ptr_val
+                            }
+                            StateAccessType::Write => {
+                                // Store the value to the local pointer created for rhs
+                                self.current_block.ins(context).store(
+                                    value_ptr_val,
+                                    rhs.expect("expecting a rhs for write"),
+                                    span_md_idx,
+                                );
+
+                                // Finally, just call state_load_quad_word/state_store_quad_word
+                                self.current_block.ins(context).state_store_quad_word(
+                                    value_ptr_val,
+                                    key_ptr_val,
+                                    span_md_idx,
+                                );
+                                rhs.expect("expecting a rhs for write")
+                            }
+                        }
+                    }
+                    _ => unreachable!(),
+                }
+            }
+            _ => unimplemented!("Other types are not yet supported in storage"),
+        })
+    }
+
+    fn get_indices_for_struct_access(
+        &self,
+        context: &Context,
+        ty: Type,
+        names: &[Ident],
+    ) -> Result<Vec<u64>, String> {
+        // Iterate over the field names from the left hand side and collect field indices.
+        Ok(names
+            .iter()
+            .fold(Ok((Vec::new(), ty)), |acc, field_name| {
+                // Make sure we have an aggregate to index into.
+                acc.and_then(|(mut fld_idcs, ty)| match ty {
+                    Type::Struct(aggregate) => {
+                        // Get the field index and also its type for the next iteration.
+                        match self
+                            .struct_names
+                            .get_aggregate_index(&aggregate, field_name.as_str())
+                        {
+                            None => Err(format!(
+                                "Unknown field name {} for struct ???",
+                                field_name.as_str()
+                            )),
+                            Some(field_idx) => {
+                                let field_type = context.aggregates[aggregate.0].field_types()
+                                    [field_idx as usize];
+
+                                // Save the field index.
+                                fld_idcs.push(field_idx);
+                                Ok((fld_idcs, field_type))
+                            }
+                        }
+                    }
+                    _otherwise => {
+                        Err("Reassignment with multiple accessors to non-aggregate.".into())
+                    }
+                })
+            })?
+            .0)
+    }
 }
 
 // -------------------------------------------------------------------------------------------------
@@ -1645,26 +1984,28 @@ fn convert_resolved_type(
         TypeInfo::Byte => Type::Uint(8), // XXX?
         TypeInfo::B256 => Type::B256,
         TypeInfo::Str(n) => Type::String(*n),
-        TypeInfo::Struct { name, fields } => match struct_names.get_aggregate_by_name(name) {
-            Some(existing_aggregate) => Type::Struct(existing_aggregate),
-            None => {
-                // Let's create a new aggregate from the TypeInfo.
-                create_struct_aggregate(context, struct_names, name.clone(), fields.clone())
-                    .map(&Type::Struct)?
+        TypeInfo::Struct { name, fields } => {
+            match struct_names.get_aggregate_by_name(name.as_str()) {
+                Some(existing_aggregate) => Type::Struct(existing_aggregate),
+                None => {
+                    // Let's create a new aggregate from the TypeInfo.
+                    create_struct_aggregate(context, struct_names, name.to_string(), fields.clone())
+                        .map(&Type::Struct)?
+                }
             }
-        },
+        }
         TypeInfo::Enum {
             name,
             variant_types,
         } => {
-            match struct_names.get_aggregate_by_name(name) {
+            match struct_names.get_aggregate_by_name(name.as_str()) {
                 Some(existing_aggregate) => Type::Struct(existing_aggregate),
                 None => {
                     // Let's create a new aggregate from the TypeInfo.
                     create_enum_aggregate(
                         context,
                         struct_names,
-                        name.clone(),
+                        name.to_string(),
                         variant_types.clone(),
                     )
                     .map(&Type::Struct)?
@@ -1837,7 +2178,7 @@ mod tests {
             print_intermediate_asm: false,
             print_finalized_asm: false,
             print_ir: false,
-            generated_names: std::sync::Arc::new(std::sync::Mutex::new(vec![])),
+            generated_names: Default::default(),
         };
 
         let mut warnings = vec![];
