@@ -1,4 +1,10 @@
-use crate::{build_config::BuildConfig, error::ok, parser::Rule, CompileResult, Expression};
+use crate::{
+    build_config::BuildConfig,
+    error::{ok, ParserLifter},
+    error_recovery_exp,
+    parser::Rule,
+    CompileResult, Expression,
+};
 
 use sway_types::span;
 
@@ -13,7 +19,7 @@ impl ReturnStatement {
     pub(crate) fn parse_from_pair(
         pair: Pair<Rule>,
         config: Option<&BuildConfig>,
-    ) -> CompileResult<Self> {
+    ) -> CompileResult<ParserLifter<Self>> {
         let span = span::Span {
             span: pair.as_span(),
             path: config.map(|c| c.path()),
@@ -24,23 +30,29 @@ impl ReturnStatement {
         let _ret_keyword = inner.next();
         let expr = inner.next();
         let res = match expr {
-            None => ReturnStatement {
-                expr: Expression::Tuple {
-                    fields: vec![],
-                    span,
-                },
-            },
-            Some(expr_pair) => {
-                let expr = check!(
-                    Expression::parse_from_pair(expr_pair, config),
-                    Expression::Tuple {
+            None => {
+                let stmt = ReturnStatement {
+                    expr: Expression::Tuple {
                         fields: vec![],
-                        span
+                        span,
                     },
+                };
+                ParserLifter::empty(stmt)
+            }
+            Some(expr_pair) => {
+                let expr_result = check!(
+                    Expression::parse_from_pair(expr_pair, config),
+                    ParserLifter::empty(error_recovery_exp(span)),
                     warnings,
                     errors
                 );
-                ReturnStatement { expr }
+                let stmt = ReturnStatement {
+                    expr: expr_result.value,
+                };
+                ParserLifter {
+                    var_decls: expr_result.var_decls,
+                    value: stmt,
+                }
             }
         };
         ok(res, warnings, errors)
