@@ -5,6 +5,7 @@ use crate::control_flow_analysis::ControlFlowGraph;
 use crate::parse_tree::{MethodName, StructExpressionField};
 use crate::parser::{Rule, SwayParser};
 use crate::semantic_analysis::TCOpts;
+use pest::iterators::Pairs;
 use pest::Parser;
 use std::collections::{HashMap, VecDeque};
 
@@ -173,11 +174,12 @@ pub(crate) fn type_check_method_application(
     // type check all of the arguments against the parameters in the method declaration
     for (arg, param) in args_buf.iter().zip(method.parameters.iter()) {
         // if the return type cannot be cast into the annotation type then it is a type error
-        match crate::type_engine::unify_with_self(
+        match unify_with_self(
             arg.return_type,
             param.r#type,
             self_type,
             &arg.span,
+            "This argument's type is not castable to the declared parameter type.",
         ) {
             Ok(mut ws) => {
                 warnings.append(&mut ws);
@@ -220,6 +222,44 @@ pub(crate) fn type_check_method_application(
                 .map(|(param, arg)| (param.name.clone(), arg))
                 .collect::<Vec<(_, _)>>();
 
+            let selector = if method.is_contract_call {
+                let contract_address = match contract_caller
+                    .map(|x| crate::type_engine::look_up_type_id(x.return_type))
+                {
+                    Some(TypeInfo::ContractCaller { address, .. }) => address,
+                    _ => {
+                        errors.push(CompileError::Internal(
+                            "Attempted to find contract address of non-contract-call.",
+                            span.clone(),
+                        ));
+                        String::new()
+                    }
+                };
+                // TODO(static span): this can be a normal address expression,
+                // so we don't need to re-parse and re-compile
+                let contract_address = check!(
+                    re_parse_expression(
+                        contract_address.into(),
+                        build_config,
+                        namespace,
+                        crate_namespace,
+                        self_type,
+                        dead_code_graph,
+                        opts,
+                    ),
+                    return err(warnings, errors),
+                    warnings,
+                    errors
+                );
+                let func_selector = check!(method.to_fn_selector_value(), [0; 4], warnings, errors);
+                Some(ContractCallMetadata {
+                    func_selector,
+                    contract_address: Box::new(contract_address),
+                })
+            } else {
+                None
+            };
+
             TypedExpression {
                 expression: TypedExpressionVariant::FunctionApplication {
                     name: CallPath {
@@ -230,44 +270,7 @@ pub(crate) fn type_check_method_application(
                     contract_call_params: contract_call_params_map,
                     arguments: args_and_names,
                     function_body: method.body.clone(),
-                    selector: if method.is_contract_call {
-                        let contract_address = match contract_caller
-                            .map(|x| crate::type_engine::look_up_type_id(x.return_type))
-                        {
-                            Some(TypeInfo::ContractCaller { address, .. }) => address,
-                            _ => {
-                                errors.push(CompileError::Internal(
-                                    "Attempted to find contract address of non-contract-call.",
-                                    span.clone(),
-                                ));
-                                String::new()
-                            }
-                        };
-                        // TODO(static span): this can be a normal address expression,
-                        // so we don't need to re-parse and re-compile
-                        let contract_address = check!(
-                            re_parse_expression(
-                                contract_address.into(),
-                                build_config,
-                                namespace,
-                                crate_namespace,
-                                self_type,
-                                dead_code_graph,
-                                opts,
-                            ),
-                            return err(warnings, errors),
-                            warnings,
-                            errors
-                        );
-                        let func_selector =
-                            check!(method.to_fn_selector_value(), [0; 4], warnings, errors);
-                        Some(ContractCallMetadata {
-                            func_selector,
-                            contract_address: Box::new(contract_address),
-                        })
-                    } else {
-                        None
-                    },
+                    selector,
                 },
                 return_type: method.return_type,
                 is_constant: IsConstant::No,
@@ -377,7 +380,7 @@ fn re_parse_expression(
         path: None,
     };
 
-    let mut contract_pairs = match SwayParser::parse(Rule::expr, contract_string) {
+    let mut contract_pairs: Pairs<Rule> = match SwayParser::parse(Rule::expr, contract_string) {
         Ok(o) => o,
         Err(_e) => {
             errors.push(CompileError::Internal(
@@ -398,15 +401,17 @@ fn re_parse_expression(
         }
     };
 
-    let contract_address = check!(
+    // purposefully ignore var_decls as those have already been lifted during parsing
+    let ParserLifter { value, .. } = check!(
         Expression::parse_from_pair(contract_pair, Some(build_config)),
         return err(warnings, errors),
         warnings,
         errors
     );
+
     let contract_address = check!(
         TypedExpression::type_check(TypeCheckArguments {
-            checkee: contract_address,
+            checkee: value,
             namespace,
             crate_namespace,
             return_type_annotation: insert_type(TypeInfo::Unknown),
