@@ -3,7 +3,7 @@ use super::*;
 use crate::{
     build_config::BuildConfig,
     parse_tree::OwnedCallPath,
-    semantic_analysis::ast_node::{OwnedTypedEnumVariant, OwnedTypedStructField},
+    semantic_analysis::ast_node::{TypedEnumVariant, TypedStructField},
     Ident, Rule, TypeParameter,
 };
 
@@ -24,12 +24,12 @@ pub enum TypeInfo {
     Str(u64),
     UnsignedInteger(IntegerBits),
     Enum {
-        name: String,
-        variant_types: Vec<OwnedTypedEnumVariant>,
+        name: Ident,
+        variant_types: Vec<TypedEnumVariant>,
     },
     Struct {
-        name: String,
-        fields: Vec<OwnedTypedStructField>,
+        name: Ident,
+        fields: Vec<TypedStructField>,
     },
     Boolean,
     /// For the type inference engine to use when a type references another type
@@ -355,7 +355,7 @@ impl TypeInfo {
                 let field_names = {
                     let names = fields
                         .iter()
-                        .map(|OwnedTypedStructField { r#type, .. }| {
+                        .map(|TypedStructField { r#type, .. }| {
                             resolve_type(*r#type, error_msg_span)
                                 .expect("unreachable?")
                                 .to_selector_name(error_msg_span)
@@ -461,7 +461,7 @@ impl TypeInfo {
             TypeInfo::Unknown
             | TypeInfo::Custom { .. }
             | TypeInfo::SelfType
-            | TypeInfo::UnknownGeneric { .. } => Err(CompileError::TypeMustBeKnown {
+            | TypeInfo::UnknownGeneric { .. } => Err(CompileError::UnableToInferGeneric {
                 ty: self.friendly_type_str(),
                 span: err_span.clone(),
             }),
